@@ -29,7 +29,7 @@ import fs.path
 import pytest
 
 from PyQt5 import QtWidgets, QtTest
-from conftest import ROOT_FS, TESTDATA_DIR, ROOT_DIR, SUB_DIRS, Dummy_Filepicker
+from conftest import ROOT_FS, TESTDATA_DIR, ROOT_DIR, SUB_DIRS, Dummy_Filepicker, force_dir
 from fslib import fs_filepicker
 from fslib.fs_filepicker import fs_filepicker as fsfp
 from fslib.fs_filepicker import main as fsmain
@@ -58,7 +58,7 @@ class Test_Open_FilePicker(object):
         self.window.action()
         filename = self.window.filename
         self.window.close()
-        assert filename is None
+        assert filename == u""
 
     def test_fs_filepicker_files_selected_ok(self):
         data_fs = fs.open_fs(fs.path.join(ROOT_DIR, TESTDATA_DIR))
@@ -126,7 +126,8 @@ class Test_Open_FilePicker(object):
         QtWidgets.QApplication.processEvents()
         self.window.action()
         QtWidgets.QApplication.processEvents()
-        assert self.window.filename == u"example.txt"
+        filename = u"{}{}{}".format(force_dir(ROOT_DIR), fs.path.forcedir(TESTDATA_DIR), u"example.txt")
+        assert self.window.filename[7:] == filename
 
     def test_subdirectory(self):
         _folder_names = [list(name)[0] for name in self.window.dir_list_items]
@@ -140,7 +141,9 @@ class Test_Open_FilePicker(object):
         self.window.action()
         QtWidgets.QApplication.processEvents()
         self.window.close()
-        assert self.window.filename == "foo.txt"
+        filename = u"{}{}{}".format(force_dir(ROOT_DIR), fs.path.forcedir(TESTDATA_DIR),
+                                    fs.path.join(self.window.selected_dir, u"foo.txt"))
+        assert self.window.filename[7:] == filename
 
     def test_onCellClicked(self):
         self.window.onCellDoubleClicked(0, 0)
@@ -164,12 +167,37 @@ class Test_Open_FilePicker(object):
         QtWidgets.QApplication.processEvents()
         assert self.window.ui_Action.isEnabled() is False
 
+    def test_selectFileinSubDir_andChangeDir(self):
+        _folder_names = [list(name)[0] for name in self.window.dir_list_items]
+        index = _folder_names.index(u'bar')
+        self.window.onCellDoubleClicked(index, 0)
+        QtWidgets.QApplication.processEvents()
+        _file_names = [list(name)[0] for name in self.window.file_list_items]
+        sel_name = _file_names[0]
+        assert sel_name == u"bar/foo.txt"
+        self.window.onCellClicked(0, 0)
+        QtWidgets.QApplication.processEvents()
+        self.window.history_previous()
+        QtWidgets.QApplication.processEvents()
+        _file_names = [list(name)[0] for name in self.window.file_list_items]
+        result = sel_name in _file_names
+        # history previous, name not there
+        assert self.window.ui_Action.isEnabled() is result
+        index = _folder_names.index(u'bar')
+        self.window.onCellDoubleClicked(index, 0)
+        QtWidgets.QApplication.processEvents()
+        _file_names = [list(name)[0] for name in self.window.file_list_items]
+        sel_name = self.window.ui_SelectedName.text()
+        assert sel_name == u""
+        assert self.window.ui_Action.isEnabled() is False
+
     def test_open_file_on_doubleClick(self):
         _names = [list(name)[0] for name in self.window.dir_list_items + self.window.file_list_items]
         index = _names.index(u'example.csv')
         self.window.onCellDoubleClicked(index, 0)
         QtWidgets.QApplication.processEvents()
-        assert self.window.filename == "example.csv"
+        filename = u"{}{}{}".format(force_dir(ROOT_DIR), fs.path.forcedir(TESTDATA_DIR), u"example.csv")
+        assert self.window.filename[7:] == filename
 
 
 class Test_Save_FilePicker(object):
@@ -199,6 +227,7 @@ class Test_Save_FilePicker(object):
         self.window.cancel()
         QtWidgets.QApplication.processEvents()
         filename = self.window.filename
+        self.window.close()
         assert filename is None
 
     def test_filename_save_newname(self):
@@ -206,23 +235,21 @@ class Test_Save_FilePicker(object):
         QtWidgets.QApplication.processEvents()
         self.window.action()
         QtWidgets.QApplication.processEvents()
-        self.window.close()
-        filename = self.window.filename
-        assert filename == u"newexample.txt"
+        filename = u"{}{}{}".format(force_dir(ROOT_DIR), fs.path.forcedir(TESTDATA_DIR), u"newexample.txt")
+        assert self.window.filename[7:] == filename
 
     def test_filename_high_light_by_selectedname(self):
-        pytest.skip("this needs further investigation")
         index = sorted(SUB_DIRS).index(fs.path.join(u"testdata", u"foo"))
         self.window.ui_FileList.selectRow(index)
         QtWidgets.QApplication.processEvents()
-        self.window.onCellClicked(index, 0)
+        self.window.onCellDoubleClicked(index, 0)
         QtWidgets.QApplication.processEvents()
         self.window.ui_SelectedName.setText(u'foo.txt')
         QtWidgets.QApplication.processEvents()
         self.window.selection_name()
         QtWidgets.QApplication.processEvents()
-        # get the highlighted row
         index = self.window.ui_FileList.selectedIndexes()[0].row()
+        self.window.close()
         assert index == 0
 
     def test_default_filename(self):
@@ -231,9 +258,8 @@ class Test_Save_FilePicker(object):
         QtWidgets.QApplication.processEvents()
         self.window.action()
         QtWidgets.QApplication.processEvents()
-        self.window.close()
-        filename = self.window.filename
-        assert filename == u"abc.txt"
+        filename = u"{}{}{}".format(force_dir(ROOT_DIR), fs.path.forcedir(TESTDATA_DIR), self.window.default_filename)
+        assert self.window.filename[7:] == filename
 
     def test_save_in_empty_dir(self):
         index = sorted(SUB_DIRS).index(fs.path.join(u"testdata", u"empty"))
@@ -286,6 +312,92 @@ class Test_Save_FilePicker(object):
         assert self.window.last_dir_index == 1
 
 
+class Test_Save_FilePicker_default(object):
+    def setup(self):
+        self.application = QtWidgets.QApplication([])
+        self.fs_url = ROOT_FS.geturl(TESTDATA_DIR)
+        self.window = fs_filepicker.FilePicker(fs_url=[self.fs_url, TESTDATA_DIR], show_save_action=True,
+                                               file_pattern=[u'All Files (*)', u'CSV Files (*.csv)'],
+                                               default_filename="result.txt")
+        self.window.show()
+        QtWidgets.QApplication.processEvents()
+        QtTest.QTest.qWaitForWindowExposed(self.window)
+        QtWidgets.QApplication.processEvents()
+
+    def teardown(self):
+        self.application.quit()
+
+    def test_default_filename(self):
+        self.window.close()
+        assert self.window.default_filename == u"result.txt"
+
+    @mock.patch("fslib.fs_filepicker.QtWidgets.QMessageBox.warning",
+                return_value=QtWidgets.QMessageBox.No)
+    @mock.patch("fslib.fs_filepicker.QtWidgets.QMessageBox.question",
+                return_value=QtWidgets.QMessageBox.Yes)
+    def test_action(self, mockwarning, mockinformation):
+        self.window.ui_SelectedName.setText(u"foo")
+        QtWidgets.QApplication.processEvents()
+        assert self.window.show_save_action is True
+        self.window.action()
+        assert QtWidgets.QMessageBox.No
+        self.window.ui_SelectedName.setText(u"example.csv")
+        QtWidgets.QApplication.processEvents()
+        self.window.action()
+        QtWidgets.QApplication.processEvents()
+        assert QtWidgets.QMessageBox.Yes
+        filename = u"{}{}{}".format(force_dir(ROOT_DIR), fs.path.forcedir(TESTDATA_DIR), u"example.csv")
+        assert self.window.filename[7:] == filename
+
+    @mock.patch("fslib.fs_filepicker.QtWidgets.QMessageBox.warning",
+                return_value=QtWidgets.QMessageBox.No)
+    @mock.patch("fslib.fs_filepicker.QtWidgets.QMessageBox.question",
+                return_value=QtWidgets.QMessageBox.Yes)
+    def test_action_in_subdir(self, mockwarning, mockinformation):
+        self.window.ui_FileList.selectRow(0)
+        QtWidgets.QApplication.processEvents()
+        self.window.onCellDoubleClicked(0, 0)
+        QtWidgets.QApplication.processEvents()
+        self.window.ui_FileList.selectRow(0)
+        QtWidgets.QApplication.processEvents()
+        self.window.onCellClicked(0, 0)
+        QtWidgets.QApplication.processEvents()
+        self.window.action()
+        QtWidgets.QApplication.processEvents()
+        assert QtWidgets.QMessageBox.Yes
+        filename = u"{}{}{}".format(force_dir(ROOT_DIR), fs.path.forcedir(TESTDATA_DIR),
+                                    fs.path.join(self.window.selected_dir, u"foo.txt"))
+        assert self.window.filename[7:] == filename
+
+
+class Test_FilePicker_dirs(object):
+    def setup(self):
+        self.application = QtWidgets.QApplication([])
+        self.fs_url = ROOT_FS.geturl(TESTDATA_DIR)
+        self.window = fs_filepicker.FilePicker(fs_url=self.fs_url, show_save_action=True, show_dirs_only=True)
+        self.window.show()
+        QtWidgets.QApplication.processEvents()
+        QtTest.QTest.qWaitForWindowExposed(self.window)
+        QtWidgets.QApplication.processEvents()
+
+    def teardown(self):
+        self.application.quit()
+
+    def test_show_action(self):
+        assert self.window.ui_Action.text() == u"Get Directory"
+        assert self.window.ui_Action.isEnabled() is True
+
+    def test_select_directory(self):
+        self.window.ui_FileList.selectRow(0)
+        QtWidgets.QApplication.processEvents()
+        self.window.onCellClicked(0, 0)
+        QtWidgets.QApplication.processEvents()
+        self.window.action()
+        filename = u"{}{}{}".format(force_dir(ROOT_DIR),
+                                    fs.path.forcedir(TESTDATA_DIR), self.window.ui_SelectedName.text())
+        assert self.window.filename[7:] == filename
+
+
 class Test_MoreUrls(object):
     def setup(self):
         self.application = QtWidgets.QApplication([])
@@ -297,11 +409,15 @@ class Test_MoreUrls(object):
         QtTest.QTest.qWaitForWindowExposed(self.window)
         QtWidgets.QApplication.processEvents()
 
+    def teardown(self):
+        self.application.quit()
+
     def test_dirs(self):
         items = []
         for index in xrange(self.window.ui_fs_serverlist.count()):
             items.append(self.window.ui_fs_serverlist.item(index).text())
         cmp_items = ','.join(items)
+        self.window.close()
         assert SUB_DIRS[0] in cmp_items.replace(u"\\", u"/")
         assert SUB_DIRS[1] in cmp_items.replace(u"\\", u"/")
         assert SUB_DIRS[2] in cmp_items.replace(u"\\", u"/")
@@ -313,6 +429,9 @@ class Test_Filepicker(object):
         self.application = QtWidgets.QApplication([])
         self.fs_url = [ROOT_FS.geturl(_dir) for _dir in SUB_DIRS]
 
+    def teardown(self):
+        self.application.quit()
+
     def test_fsfp_nothing_selected(self):
         result = fsfp(parent=None, fs_url=self.fs_url)
         assert result == (None, None)
@@ -326,10 +445,66 @@ class Test_Filepicker(object):
     @mock.patch("fslib.fs_filepicker.FilePicker", return_value=fp)
     def test_fsfp_fileselected(self, mockresult):
         fn, tf = fsfp(parent=None, fs_url=self.fs_url)
-        assert fn.startswith(u'file://')
-        assert u'testdata/foo/foo.txt' in fn
+        assert fn == u"foo.txt"
         assert tf == 'All Files (*)'
 
     @mock.patch("fslib.fs_filepicker.fs_filepicker", return_value=[None])
     def test_main_call(self, mockresult):
+        pytest.skip("won't work with coverage")
         assert fsmain() is None
+
+
+class Test_Navigation(object):
+    def setup(self):
+        self.other_fs_url = [ROOT_FS.geturl(_dir) for _dir in SUB_DIRS]
+        self.application = QtWidgets.QApplication([])
+        self.fs_url = ROOT_FS.geturl(TESTDATA_DIR)
+        self.window = fs_filepicker.FilePicker(fs_url=self.fs_url)
+        self.window.show()
+        QtWidgets.QApplication.processEvents()
+        QtTest.QTest.qWaitForWindowExposed(self.window)
+        QtWidgets.QApplication.processEvents()
+
+    def teardown(self):
+        self.application.quit()
+
+    @mock.patch("fslib.fs_filepicker.QtWidgets.QInputDialog.getText",
+                return_value=(ROOT_FS.geturl(TESTDATA_DIR), True))
+    def test_other_fs_button(self, mockinput):
+        self.window.other_fs_button()
+        items = [self.window.ui_fs_serverlist.item(index).text()
+                 for index in xrange(self.window.ui_fs_serverlist.count())]
+        self.window.close()
+        assert self.fs_url in items
+
+    @mock.patch("fslib.fs_filepicker.QtWidgets.QInputDialog.getText",
+                return_value=(ROOT_FS.geturl(TESTDATA_DIR), True))
+    def test_fs_select_other(self, mockinput):
+        self.window.other_fs_button()
+        items = [self.window.ui_fs_serverlist.item(index).text()
+                 for index in xrange(self.window.ui_fs_serverlist.count())]
+        index = items.index(self.fs_url)
+        self.window.ui_fs_serverlist.setCurrentRow(index)
+        QtWidgets.QApplication.processEvents()
+        self.window.fs_select_other()
+        self.window.close()
+        assert self.window.wparm is None
+        assert self.window.fs._closed is False
+
+    @mock.patch("fslib.fs_filepicker.QtWidgets.QInputDialog.getText",
+                return_value=(ROOT_FS.geturl(TESTDATA_DIR), True))
+    @mock.patch("fslib.fs_filepicker.QtWidgets.QMessageBox.information",
+                return_value=(QtWidgets.QMessageBox.Ok))
+    def test_fs_select_other_context(self, mockinput, mockremove):
+        self.window.other_fs_button()
+        items = [self.window.ui_fs_serverlist.item(index).text()
+                 for index in xrange(self.window.ui_fs_serverlist.count())]
+        assert self.fs_url in items
+        _index = items.index(self.fs_url)
+        self.window.ui_fs_serverlist.setCurrentRow(_index)
+        QtWidgets.QApplication.processEvents()
+        self.window.fs_select_other_context()
+        items = [self.window.ui_fs_serverlist.item(index).text()
+                 for index in xrange(self.window.ui_fs_serverlist.count())]
+        self.window.close()
+        assert self.fs_url not in items
